@@ -4,13 +4,14 @@ import snow_pyrepl as pyrepl
 from typing import Optional
 from replit.database import Database
 import getpass
+from datauri import DataURI
 from watchdog.observers import Observer
 from watchdog.events import LoggingEventHandler
 
-
-__version__ = "1.2.0"
+__version__ = "1.2.2"
 homedir = Path.home()
 homedir = str(homedir).replace("\\", "/")
+
 try:
 	__sid__ = open(f"{homedir}/replit-cli/connect.sid", "r").read().strip()
 except:
@@ -32,6 +33,122 @@ def get_json(user, replname, sid):
 	r = requests.get(f"https://replit.com/data/repls/@{user}/{replname}", headers=headers, cookies=cookies)
 	return r.json()['id']
 
+
+def downloadRepl(user, replname):
+	def clean(output, join=False):
+		outputList = output
+		try:
+			if join:
+				output = str(output).split('output: "')[1].split("\"\n, channel:")[0] + str(output).split('output: "')[-1].split("\"\n, channel:")[0]
+			else:
+				output = str(output).split('output: "')[1].split("\"\n, channel:")[0]
+		except:
+			output = ""
+		output = bytes(output, "utf-8").decode("unicode_escape").strip()
+		return output
+	
+	key = __sid__
+	
+	token, url = pyrepl.get_token(user, replname, key)
+	id = get_json(user, replname, key)
+	
+	client = pyrepl.Client(token, id, url)
+	print("Client instantiated")
+	channel = client.open("exec", 'execer')
+	print("Channel opened")
+	
+	files = channel.get_output({
+		"exec": {
+			"args": ["ls"]
+		}
+	})
+	
+	files = clean(files).split("\n")
+	if "__pycache__" in files:
+		files.remove("__pycache__")
+	
+	def getSubFiles(dir):
+		files = channel.get_output({
+			"exec": {
+				"args": ["sh", "-c", f"cd {dir}; ls; cd .."]
+			}
+		})
+	
+		files = clean(files).split("\n")
+		toReturn = []
+		for file in files:
+			if not file: continue
+			toReturn.append(dir+"/"+file)
+	
+		return toReturn
+	
+	
+	def isDir(file):
+		output = channel.get_output({
+			"exec": {
+				"args": ["sh", "-c", f"if [ -d '{file}' ]; then echo 'yes'; else echo 'no'; fi"]
+			}
+		})
+		
+		output = clean(output)
+		if 'yes' in output:
+			return True		
+		return False
+	
+	for file in files:
+		if not file: continue
+		newfile = file
+		ThisIsDir = isDir(newfile)
+		if ThisIsDir:
+				for file in getSubFiles(newfile):
+					files.append(file.replace("\\", "/"))
+					file = file.replace("\\", "/")
+	
+		if not ThisIsDir:
+			print(f"REFRESHING FILE/DIR FROM SERVER - {newfile}")
+	
+			if ".png" in newfile or ".jpg" in newfile or ".webp" in newfile or ".gif" in newfile or ".jpeg" in newfile:
+				ext = newfile.split(".")[-1]
+				cmd = """bash -c 'function img-data() {   TYPE=$(file --mime-type -b "$1");   ENC=$(base64 "$1");   echo "data:$TYPE;base64,$ENC"; }; img-data \""""+newfile+"""\"'"""
+				content = channel.get_output({
+					"exec": {
+						"args": ["bash", "-c", cmd]
+					}
+				})
+	
+			else:
+				content = channel.get_output({
+					"exec": {
+						"args": ["sh", "-c", f"cat '{newfile}'"]
+					}
+				})
+	
+			original = content
+			content = clean(content)
+	
+			filename = user+"/"+replname+"/"+newfile
+			os.makedirs(os.path.dirname(filename), exist_ok=True)
+	
+			if ".png" in newfile or ".jpg" in newfile or ".webp" in newfile or ".gif" in newfile or ".jpeg" in newfile:
+				try:
+					uri = DataURI(content)
+				except Exception as e:
+					content = clean(original, join=True)
+					uri = DataURI(content)
+	
+					
+				fd = open(filename, 'wb')
+				fd.write(uri.data)
+				fd.close()
+	
+			else:
+				with open(filename, "w+") as f:
+					f.write(content)
+					
+	client.close()
+	print("Client closed")
+
+
 @app.callback()
 def callback(ctx: typer.Context):
 	cmd = ctx.invoked_subcommand
@@ -53,15 +170,9 @@ def clone(repl:str=typer.Argument("", help="The URL of the repl which is to be c
 
 	user = repl.split("/")[0]
 	replname = repl.split("/")[1]
-	slug = user + "/" + replname
-	sid = __sid__
-	zip = requests.get(f"https://replit.com/@{slug}.zip", cookies={"connect.sid": sid})
-	f = open(f"{replname}.zip", "wb")
-	f.write(zip.content)
-	f.close()
-	with zipfile.ZipFile(f"{replname}.zip", 'r') as zip_ref:
-		zip_ref.extractall(replname)
-	os.remove(f"{replname}.zip")
+	os.mkdir(replname)
+	os.system("cd "+replname)
+	downloadRepl(user, replname)
 	f = open(f"{replname}/.replitcliconfig", "w")
 	r = requests.get(f"https://replit.com/data/repls/@{slug}", cookies={"connect.sid": sid}).json()
 	id = r['id']
@@ -76,22 +187,18 @@ def pull(override:bool=False):
 		content = f.read()
 		f.close()
 		url = content.split("=")[1].split("\n")[0]
-		sid = __sid__.strip()
-		zip = requests.get(f"{url}.zip", cookies={"connect.sid": sid})
-		f = open(url.split("/")[-1]+".zip", "wb")
-		f.write(zip.content)
-		f.close()
-		with zipfile.ZipFile(url.split("/")[-1]+".zip", 'r') as zip_ref:
-			files = glob.glob("*")
-			for file in files:
-				if file != url.split("/")[-1]+".zip":
-					if not "." in file:
-						shutil.rmtree(file)
-					else:
-						os.remove(file)
-			zip_ref.extractall("")
-		os.remove(url.split("/")[-1]+".zip")
-		name = url.split("/")[-1]
+		slug = url.split("/@")[-1]
+		user = slug.split("/")[0]
+		replname = slug.split("/")[1]
+		files = glob.glob("*")
+		for file in files:
+			try:
+				shutil.rmtree(file)
+			except:
+				os.remove(file)
+
+		downloadRepl(user, replname)
+
 		f = open(f".replitcliconfig", "w")
 		slug = url.split("/@")[-1]
 		r = requests.get(f"https://replit.com/data/repls/@{slug}", cookies={"connect.sid": sid}).json()
@@ -106,43 +213,10 @@ def pull(override:bool=False):
 		content = f.read()
 		f.close()
 		url = content.split("=")[1].split("\n")[0]
-		sid = __sid__.strip()
 		slug = url.split("/@")[-1]
-		zip = requests.get(f"{url}.zip", cookies={"connect.sid": sid})
-		f = open(url.split("/")[-1]+".zip", "wb")
-		f.write(zip.content)
-		f.close()
-		with zipfile.ZipFile(url.split("/")[-1]+".zip", 'r') as zip_ref:
-			zip_ref.extractall(".temp")
-		os.remove(url.split("/")[-1]+".zip")
-		files = glob.glob("*")
-		files2 = glob.glob(".*")
-		for file in files2:
-			files.append(file)
-		done = []
-		for file in files:
-			# print(f"FILE/DIR FOUND - {file}")
-			if "connect.sid" not in file and ".replitcliconfig" not in file:
-				newfile = file
-				if "." not in file:
-					newfile = file + "/"
-
-				if "." not in newfile:
-					filelist = glob.glob(f"{newfile}*")
-					for file in filelist:
-						files.append(file.replace("\\", "/"))
-						file = file.replace("\\", "/")
-
-				if "." in newfile:
-					try:
-						output = open(f".temp/{newfile}", "r").read()
-						f = open(newfile, "w")
-						f.write(output)
-						f.close()
-					except:
-						output = None
-					done.append(newfile)
-		shutil.rmtree(".temp")
+		user = slug.split("/")[0]
+		replname = slug.split("/")[1]
+		downloadRepl(user, replname)
 	typer.echo(f"Refreshed Local Repl Dir")
 
 
@@ -507,15 +581,8 @@ def shell(repl:str):
 		output = output2
 		output = bytes(output, "utf-8").decode("unicode_escape")
 		print(output)
-	try:
-		channel2 = client.open("exec", "runner")
-		channel2.get_output({
-			"exec": {
-				"args": ['kill', '1']
-			}
-		})
-	except:
-		pass
+
+	client.close()
 
 @app.command(help="Execute a command to run on the remote repl.")
 def exec(repl:str, cmd:str):
@@ -541,15 +608,7 @@ def exec(repl:str, cmd:str):
 	output = output2
 	output = bytes(output, "utf-8").decode("unicode_escape")
 	print(output)
-	try:
-		channel2 = client.open("exec", "runner")
-		channel2.get_output({
-			"exec": {
-				"args": ['kill', '1']
-			}
-		})
-	except:
-		pass
+	client.close()
 
 @app.command(help="Interact with the Environment of the Repl of the Current Working Directory.")
 def env(contents:bool=True, key:str="", value:str="", delete:str=""):
